package bsp

import (
	"fmt"
	"sync"

	"tuxpa.in/t/wm/src/bsp/cfg"
)

type WM struct {
	Desktops []*Desktop
	Monitors []*Monitor

	Cfg *cfg.Modifier[*Config]

	mu sync.RWMutex
}

type Desktop struct {
	Name    string
	Monitor *Monitor

	Cfg *cfg.Modifier[*Config]
}

type Monitor struct {
	Name string

	Cfg *cfg.Modifier[*Config]
}

type Node struct {
	Name string

	Cfg *cfg.Modifier[*Config]
}

func (w *WM) Mutate(fn func() error) error {
	w.mu.Lock()
	defer w.mu.Unlock()
	if fn != nil {
		return fn()
	}
	return nil
}

func (w *WM) View(fn func() error) error {
	w.mu.RLock()
	defer w.mu.RUnlock()
	if fn != nil {
		return fn()
	}
	return nil
}

func (w *WM) AddMonitor(name string) error {
	return w.Mutate(func() error {
		w.Monitors = append(w.Monitors, &Monitor{
			Name: name,
			Cfg:  cfg.NewModifier(&Config{}),
		})
		return nil
	})
}
func (w *WM) AddDesktop(name string, monitorName string) error {
	return w.Mutate(func() error {
		var monitor *Monitor
		for _, v := range w.Monitors {
			if v.Name == monitorName {
				monitor = v
			}
		}
		if monitor == nil {
			return fmt.Errorf("invalid descriptor found in '%s'", monitorName)
		}
		w.Desktops = append(w.Desktops, &Desktop{
			Name:    name,
			Cfg:     cfg.NewModifier(&Config{}),
			Monitor: monitor,
		})
		return nil
	})
}

func NewWM() *WM {
	w := &WM{
		Cfg: cfg.NewModifier(&Config{}),
	}
	w.Cfg.FillDefaults()

	return w
}
